#!/usr/bin/env python
#
import os
import sys
import optparse
#import time
import serial
import xml.etree.ElementTree as ElementTree
import cherrypy
#import profile
#
# Data browser class
#
class DataBrowser():
#
    title = 'Avanta heater'
#
# Header
#
    def __header__(self):
        output = '''
            <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
            <html>
            <head>
                <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                <title>%s</title>
            </head>
            <body>
            <center>
            <h1>%s</h1>
            </center>
            ''' % (self.title, self.title)
        return output
    __header__.exposed = False
#
# Footer
#
    def __footer__(self):
        output = '''
            <br>
            <center>
            <footer>
                footer
            </footer>
            </center>
            </body>
            </html>'''
        return output
    __footer__.exposed = False
#
# View tables
#
    def tables(self, *args):
        TableCount = 0
        TableRowColered = False
#
# Header
#
        output = self.__header__()
#
# Table(s)
#
        output += '''<center>'''
        for Item in Avanta.GetValues(options.catagory, 'all', False, 1):
            if len(Item) == 1:
                if TableCount != 0:
                    output += '''
                        </table>
                        <br>'''
                TableCount += 1
                output += '''
                    <table border=\"1\" cellspacing=\"0\" width=\"600\">
                        <tr bgcolor=\"#FFCC7F\" >
                            <th colspan=\"3\">%s</th>
                        </tr>''' % Item[0]
                TableRowColered = False
            else:
                if TableRowColered == True:
                    output += '''<tr bgcolor=\"#E5F3FF\">'''
                else:
                    output += '''<tr>'''
                TableRowColered = not TableRowColered
                output += '''
                            <td width=\"45%%\">&nbsp;&nbsp;%s</td>
                            <td width=\"35%%\">&nbsp;&nbsp;%s</td>
                            <td width=\"20%%\">&nbsp;&nbsp;%s</td>
                        </tr>''' % (Item[0], Item[1], Item[2])
        output += '''</table>'''
        output += '''</center>'''
#
# Footer
#
        output += self.__footer__()
        return output
    tables.exposed = True
#
# Remeha heater class
#
class RemehaHeater():
#
# Init heater: Get properties from Remeha xml files and open port
#
    def __init__(self, address):
        self.LanguageXML = ElementTree.parse(options.config + '/Language-NL.xml')
        self.ConfigXML = ElementTree.parse(options.config + '/AvantaV1_P3-Config.xml')
        self.RequestXML = ElementTree.parse(options.config + '/AvantaV1_P3-Request.xml')
        self.Heater = serial.serial_for_url(address, timeout=1)
        self.Debug = 0
#
# Close heater
#
    def __del__(self):
        self.Heater.close()
#
# Get text from languages file
#
    def __GetText__(self, Id):
        for Item in self.LanguageXML.findall('text'):
            if Item.attrib.get('id') == Id:
                break
        if Item.attrib.get('value') == 'end of file':
            return ''
        else:
            return Item.attrib.get('value')
#
# Read the data package for a specific category
#
    def __GetDataPackage__(self, Category):
        DataPackage = ''
        for Item in self.RequestXML.findall('request'):
            if Item.attrib.get('id') == Category:
                Size = int(Item.attrib.get('size'))
                for String in Item.findall('string'):
                    RequestString = ''
                    for Char in String.attrib.get('hex').split():
#                        print Char, hex(int(Char, 16)), chr(int(Char, 16))
                        RequestString = RequestString + chr(int(Char, 16))
#                    print RequestString
                    self.Heater.write(RequestString)
                    Data = self.Heater.read(Size)
                    if Size == 24:
                        DataPackage = DataPackage + Data[6:22]
                    elif Size == 64:
                        DataPackage = DataPackage + Data[6:62]
                    else:
                        print 'Error: Unknown data package size' 
        return DataPackage
#
# Get values from the data package for a specific category
#
    def __GetValuesFromDataPackage__(self, DataPackage, Category, Parameter, Raw, NDVU):
        if self.Debug == 1:
            print 'Category =', Category
            print 'Raw =', Raw
            print 'NDVU =', NDVU
        if NDVU == 1:
            Values = [[str.capitalize(Category)]]
        else:
            Values = []
        for Config in self.ConfigXML.findall('configurations'):
            if Category == Config.attrib.get('name') or Category == 'all':
                for Param in Config.findall('config'):
                    Name = self.__GetText__(Param.find('present').attrib.get('name.nr'))
                    if Parameter == Name or Parameter == 'all':
                        if NDVU == 1:
                            Description = self.__GetText__(Param.find('present').attrib.get('description.nr'))
                            Unit = self.__GetText__(Param.find('present').attrib.get('unit.nr'))
                        Function = Param.find('function')
                        Byte = int(Function.attrib.get('byte'))
                        Invert = False
                        Value = ''
                        try:
                            Expression = Function.attrib.get('expression')
                        except:
                            Expression = ''
                        try:
                            Format = Function.attrib.get('format')
                        except:
                            Format = ''
                        try:
                            Bit = int(Function.attrib.get('bit'))
                        except:
                            Bit = -1
                        try:
                            Invert = Function.attrib.get('invert')
                        except:
                            Invert = 'False'
                        try:
                            Number = int(Function.attrib.get('number'))
                        except:
                            Number = ''
#
#                       Interpret function
#
                        if Bit != -1:
                            Value = ord(DataPackage[Byte]) & pow(2,Bit)
                            if self.Debug == 1:
                                byte2bits = [''.join(['01'[i&(1<<b)>0] for b in xrange(7,-1,-1)]) for i in xrange(256)]
                                print 'pow(2,Bit), Bit   :',byte2bits[pow(2,Bit)], Bit
                                print 'Value             :',byte2bits[ord(DataPackage[Byte])]
                                print 'Value & Pow(2,Bit):',byte2bits[Value]
                            if Value != 0:
                                Value = 1
                            if Invert == 'true' or Invert == 'True':
                                Value = abs(Value - 1)
                        elif Expression == 'A':
                            Value = ord(DataPackage[Byte])
                        elif Expression == 'A x 100':
                            Value = ord(DataPackage[Byte])*100
                        elif Expression == 'sgn(A.0 + B.1) x 0.01':
                            Value = float(ord(DataPackage[Byte]) + ord(DataPackage[Byte + 1])*256)*0.01
                        elif Expression == 'A.0 + B.1':
                            Value = int(ord(DataPackage[Byte]) + ord(DataPackage[Byte + 1])*256)
                        elif Expression == 'A x 0.1':
                            Value = float(ord(DataPackage[Byte]))*0.1
                        elif Expression == '(A.1 + B.0) x 1':
                            Value = int(ord(DataPackage[Byte])*256 + ord(DataPackage[Byte + 1]))
                        elif Expression == '(A.1 + B.0) x 2':
                            Value = int(ord(DataPackage[Byte])*256 + ord(DataPackage[Byte + 1]))*2
                        elif Expression == '(A.1 + B.0) x 8':
                            Value = int(ord(DataPackage[Byte])*256 + ord(DataPackage[Byte + 1]))*8
                        elif Number == 5:
                            Value = ''
                            for i in range(Byte, Byte + 5):
                                if DataPackage[i].isalpha():
                                    Value = Value + DataPackage[i]
                                else:
                                    Value = Value + hex(ord(DataPackage[i]))[2:4]
                        elif Number == 6:
                            Value = float(hex(ord(DataPackage[Byte]))[2:4])/10
                        else:
                            Value = 'Error: Unknown function'
#
#                       Check value
#
                        try:
                            Limits = Param.find('limits')
                            try:
                                Min = float(Limits.attrib.get('min'))
                                if float(Value) < Min:
                                    Value = 'out of range (min)'
                            except:
                                pass
                            try:
                                Max = float(Limits.attrib.get('max'))
                                if float(Value) > Max:
                                    Value = 'out of range (max)'
                            except:
                                pass
                        except:
                            pass
#
#                       Convert value to text
#
                        if Raw == False:
                            try:
                                NodeRef = Param.find('node-ref')
                                Type = NodeRef.attrib.get('type')
                                Id = NodeRef.attrib.get('id')
                                for Selection in self.ConfigXML.findall(Type):
                                    if Selection.attrib.get('id') == Id:
                                        for SelectionValue in Selection.findall('selection'):
                                            if SelectionValue.attrib.get('value') == str(Value):
                                                Value = self.__GetText__(SelectionValue.attrib.get('result.nr'))
                                                break
                            except:
                                pass
                            try:
                                Selections = Param.find('selections')
                                for Selection in Selections.findall('selection'):
                                    if Selection.attrib.get('value') == str(Value):
                                        Value = self.__GetText__(Selection.attrib.get('result.nr'))
                                        break
                            except:
                                pass
                        if self.Debug == 1:
                            print Byte, Bit, Expression, Name, '=', Value, Unit
                        if NDVU == 1:
                            Values = Values + [[Name, Value, Unit, Description]]
                        else:
                            Values = Values + [Value]
        return Values
#
# Read data package for a specific category and convert data to physical values
#
    def GetValues(self, Category, Parameter, Raw, NDVU):
        if Category == 'all':
            Values = []
            for Item in self.RequestXML.findall('request'):
                Id = Item.attrib.get('id')
                DataPackage = self.__GetDataPackage__(Id)
                Values = Values + self.__GetValuesFromDataPackage__(DataPackage, Id, Parameter, Raw, NDVU) 
        else:
            DataPackage = self.__GetDataPackage__(Category)
            Values = self.__GetValuesFromDataPackage__(DataPackage, Category, Parameter, Raw, NDVU) 
        return Values
#
# Print values to stdout
#
    def PrintValues(self, Category):
        for Item in self.GetValues(Category, 'all', False, 1):
            if len(Item) == 1:
                print '\n====', Item[0], '===='
            else:
                print Item[0], '=', Item[1], Item[2]
#
# Main program
#
# Get absolute path to application
#
AppHomeDir = os.path.dirname(os.path.abspath(__file__))
#
# Init command line interpeter
#
parser = optparse.OptionParser(usage='%prog [options]\n\nRead data from a Remeha heater.')
parser.add_option('-C', '--config', action='store', type='string', dest='config', default=os.path.join(AppHomeDir,'config'),
                  help='specify the location of the configuration files (Language-NL.xml, AvantaV1_P3-Config.xml and AvantaV1_P3-Request.xml)')
parser.add_option('-c', '--catagory', action='store', type='string', dest='catagory', default='all',
                  help='specify catagory of the data set to be read. Valid catagories are: identification, sample, counter, parameter, failure or all (identification, sample, counter, parameter and failure)')
parser.add_option('-p', '--print', action='store_true', dest='stdout',
                  help='write data to standard output')
parser.add_option('-w', '--webserver', action='store_true', dest='webserver',
                  help='start web server at localhost, port 8080')
(options, args) = parser.parse_args()
#
# Connect heater
#
Avanta = RemehaHeater("rfc2217://xxx<fill in the IP address of the serial to ethernet converter>xxx")
#
# Start action
#
if options.stdout:
    Avanta.PrintValues(options.catagory)
if options.webserver:
    cherrypy.quickstart(DataBrowser())
#
# Disconnect heater
#
del Avanta
#
# Terminate
#
quit()
